var RuntimeEnvironment = {
    Set: function (target, key, value) {
        target[key] = value;
    },
    Delete: function (target, key) {
        delete target[key];
    },
    AttachToVue: function (Vue) {
        this.Set = Vue.$set;
        this.Delete = Vue.$delete;
    }
};

var Generic;
(function (Generic) {
    /**
     * Find the common value of provided array; for object array, first object is used as key reference, only property with common parts has key in result object.
     */
    function Common(values, eps) {
        if (eps === void 0) { eps = 0; }
        if (values.length > 1) {
            var reference_1 = values[0];
            if (IsObject(reference_1) && !IsArray(reference_1)) {
                if (!values.some(function (value) { return value == null || !IsObject(value); })) {
                    var common_1 = {};
                    Object.keys(reference_1).forEach(function (key) {
                        var subCommon = Common(values.map(function (value) { return value[key]; }), eps);
                        if (subCommon != null) {
                            common_1[key] = subCommon;
                        }
                        else if (values.every(function (value) { return value[key] !== undefined; })) {
                            common_1[key] = null;
                        }
                    });
                    return common_1;
                }
                else
                    return null;
            }
            else {
                return values.every(function (value) { return Equals(value, reference_1, eps); }) ? reference_1 : null;
            }
        }
        else
            return values[0];
    }
    Generic.Common = Common;
    function IsArray(target) {
        return Array.isArray(target);
    }
    Generic.IsArray = IsArray;
    function IsObject(target) {
        return target === Object(target);
    }
    Generic.IsObject = IsObject;
    function IsNumber(target) {
        return typeof target === 'number';
    }
    Generic.IsNumber = IsNumber;
    function IsPlainObject(target) {
        return IsObject(target) && target.constructor.name === 'Object';
    }
    Generic.IsPlainObject = IsPlainObject;
    function IsString(target) {
        return typeof target === 'string';
    }
    Generic.IsString = IsString;
    function IsInstance(target) {
        return IsObject(target) && target.constructor.name !== 'Object';
    }
    Generic.IsInstance = IsInstance;
    /**
     * IsEmpty() determines whether the target is empty (array of zero length, object without any property, string of zero length, null, undefined or class without any members).
     */
    function IsEmpty(target) {
        return target == null || (IsObject(target) && Object.keys(target).length === 0) || (IsString(target) && target.length === 0);
    }
    Generic.IsEmpty = IsEmpty;
    /**
     * Clone() makes a deep/shallow value copy of given object.
     */
    function Clone(target, deep) {
        if (deep === void 0) { deep = true; }
        if (Array.isArray(target)) {
            var clone_1 = target.slice();
            if (deep) {
                clone_1.forEach(function (value, index) {
                    clone_1[index] = Clone(value);
                });
            }
            return clone_1;
        }
        else if (IsObject(target)) {
            // forEach is not called for empty object, so it's logically safe to force a NonEmptyObject guard here.
            var clone_2 = Object.assign({}, target);
            if (deep) {
                Object.entries(target).forEach(function (_a) {
                    var key = _a[0], value = _a[1];
                    clone_2[key] = Clone(value);
                });
            }
            return clone_2;
        }
        else
            return target;
    }
    Generic.Clone = Clone;
    /** Null() return a null version of give object or value. e.g. 1 => null; {a : { b : 1 } } = > { a: { b: null } } */
    function Null(target) {
        return Assign(target, null);
    }
    Generic.Null = Null;
    /** Assign() return version of and index copy of given object or value filled with specified value. e.g. (1, v) => v; ({a : { b : 1 } }, v) = > { a: { b: v } } */
    function Assign(target, value) {
        if (IsObject(target)) {
            if (IsArray(target)) {
                return target.map(function (t) { return Assign(t, value); });
            }
            else if (IsObject(target)) {
                // Agian, forEach is not called for empty object, so I forced a NonEmptyObject guard.
                var nul_1 = {};
                Object.entries(target).forEach(function (_a) {
                    var key = _a[0], subVal = _a[1];
                    nul_1[key] = Assign(subVal, value);
                });
                return nul_1;
            }
        }
        else {
            return value;
        }
    }
    Generic.Assign = Assign;
    /**
     * Equals() does value compare for two arbitrary operands.
     */
    function Equals(o1, o2, eps) {
        if (eps === void 0) { eps = 0; }
        if (IsObject(o1) && IsObject(o2)) {
            if (IsArray(o1)) {
                if (IsArray(o2) && o1.length === o2.length) {
                    return o1.every(function (value, index) { return Equals(value, o2[index]); });
                }
            }
            else {
                return Object.keys(o1).length === Object.keys(o2).length && Object.keys(o1).every(function (key) { return Equals(o1[key], o2[key]); });
            }
        }
        else if (IsNumber(o1) && IsNumber(o2)) {
            return Math.abs(o1 - o2) <= eps;
        }
        return o1 === o2;
    }
    Generic.Equals = Equals;
})(Generic || (Generic = {}));

var DirectMap;
(function (DirectMap) {
    /**
     * DoubleMap() creates a new directmap with the results of calling a provided function on every key-value pair in the calling directmap.
    */
    function DoubleMap(directMap, mapping) {
        var newMap = {};
        Object.keys(directMap).forEach(function (oldKey) {
            var _a = mapping(directMap[oldKey], oldKey, directMap), key = _a.key, value = _a.value;
            newMap[key] = value;
        });
        return newMap;
    }
    DirectMap.DoubleMap = DoubleMap;
    function Every(directMap, predicate) {
        return Object.keys(directMap).every(function (key) { return predicate(directMap[key], key, directMap); });
    }
    DirectMap.Every = Every;
    function Filter(directMap, predicate) {
        var resultMap = {};
        Object.keys(directMap).forEach(function (key) {
            if (predicate(directMap[key], key, directMap)) {
                resultMap[key] = directMap[key];
            }
        });
        return resultMap;
    }
    DirectMap.Filter = Filter;
    function Find(directMap, predicate) {
        var key = Object.keys(directMap).find(function (key) { return predicate(directMap[key], key, directMap); });
        if (key) {
            return directMap[key];
        }
        else {
            return undefined;
        }
    }
    DirectMap.Find = Find;
    function FindKey(directMap, predicate) {
        return Object.keys(directMap).find(function (key) { return predicate(directMap[key], key, directMap); });
    }
    DirectMap.FindKey = FindKey;
    function Foreach(directMap, action) {
        Object.keys(directMap).forEach(function (key) {
            action(directMap[key], key, directMap);
        });
    }
    DirectMap.Foreach = Foreach;
    /**
     * From() create a directmap from a value array and a generator function for each seed.
     */
    function From(values, generator) {
        var map = {};
        values.forEach(function (seed, index) {
            var _a = generator(seed, index), key = _a.key, value = _a.value;
            map[key] = value;
        });
        return map;
    }
    DirectMap.From = From;
    /**
     * ValueMap() method creates a new directmap with the results of calling a provided function on every key-value pair in the calling directmap.
     */
    function ValueMap(directMap, mapping) {
        var newMap = {};
        Object.keys(directMap).forEach(function (key) {
            newMap[key] = mapping(directMap[key], key, directMap);
        });
        return newMap;
    }
    DirectMap.ValueMap = ValueMap;
    /**
     * Pack() packs a key and a value to a directmap.
    */
    function Pack(key, value) {
        var pack = {};
        pack[key] = value;
        return pack;
    }
    DirectMap.Pack = Pack;
    function Slice(directMap, keys) {
        return From(keys, function (key) { return ({ key: key, value: directMap[key] }); });
    }
    DirectMap.Slice = Slice;
    function Some(directMap, predicate) {
        return Object.keys(directMap).some(function (key) { return predicate(directMap[key], key, directMap); });
    }
    DirectMap.Some = Some;
    function Splice(directMap, keys) {
        var slice = {};
        keys.forEach(function (key) {
            slice[key] = directMap[key];
            RuntimeEnvironment.Delete(directMap, key);
        });
        return slice;
    }
    DirectMap.Splice = Splice;
    /**
     * ToMap() converts directmap to es6 map.
    */
    function ToMap(map) {
        var newMap = new Map();
        Foreach(map, function (value, key) { return newMap.set(key, value); });
        return newMap;
    }
    DirectMap.ToMap = ToMap;
})(DirectMap || (DirectMap = {}));

var DirectTree;
(function (DirectTree) {
    DirectTree.DEFAULT_SEPERATOR = '/';
    /**
     * Traverse the tree to apply action.
    */
    function TraverseLeaf(directTree, action, currentPath, seperator) {
        if (currentPath === void 0) { currentPath = ''; }
        if (seperator === void 0) { seperator = DirectTree.DEFAULT_SEPERATOR; }
        if (Generic.IsObject(directTree)) {
            var prefix_1 = currentPath.length > 0 ? "" + currentPath + seperator : '';
            Object.keys(directTree).forEach(function (key) {
                TraverseLeaf(directTree[key], action, "" + prefix_1 + key, seperator);
            });
        }
        else {
            action(directTree, currentPath);
        }
    }
    /**
     * Delete() deletes the subtree with provided path.
     */
    function Delete(directTree, path) {
        var branch = GetBranch(directTree, path);
        if (branch) {
            RuntimeEnvironment.Delete(branch.parent, branch.key);
        }
    }
    DirectTree.Delete = Delete;
    /**
     * Create a flat direct map of leaves with each leaf's path as it's new key.
     * @param prefix prefix of each path.
     */
    function Flat(directTree, prefix, seperator) {
        if (prefix === void 0) { prefix = ''; }
        if (seperator === void 0) { seperator = DirectTree.DEFAULT_SEPERATOR; }
        var flat = {};
        TraverseLeaf(directTree, function (value, path) {
            flat[path] = value;
        }, prefix, seperator);
        return flat;
    }
    DirectTree.Flat = Flat;
    /**
     * GetBranch() gets the parent of provided path and it's branch key.
     */
    function GetBranch(directTree, path) {
        if (IsFlat(directTree)) {
            return { parent: directTree, key: path };
        }
        else {
            var keys = path.split(DirectTree.DEFAULT_SEPERATOR);
            for (; keys.length > 1 && directTree; directTree = directTree[keys.splice(0, 1)[0]]) { }
            return (keys.length === 1 && directTree) ? { parent: directTree, key: keys[0] } : null;
        }
    }
    DirectTree.GetBranch = GetBranch;
    /**
     * Get() returns the subtree or leaf with provided path.
     */
    function Get(directTree, path) {
        if (path && path.length > 0) {
            var branch = GetBranch(directTree, path);
            return branch ? branch.parent[branch.key] : undefined;
        }
        else
            return directTree;
    }
    DirectTree.Get = Get;
    /**
     * IsFlat() determines whether a tree has been flatten.
     */
    function IsFlat(directTree) {
        var keys = Object.keys(directTree);
        return keys.length > 0 && keys.some(function (key) { return key.includes(DirectTree.DEFAULT_SEPERATOR); });
    }
    DirectTree.IsFlat = IsFlat;
    /**
     * MergeAt() merges another tree into tree's branch.
     */
    function MergeAt(directTree, path, anotherTree) {
        var branch = GetBranch(directTree, path);
        if (branch != null) {
            var target = branch.parent[branch.key];
            if (Generic.IsObject(target)) {
                MergeInternal(target, anotherTree);
            }
            else {
                MergeInternal(branch.parent, DirectMap.Pack(branch.key, anotherTree));
            }
        }
    }
    DirectTree.MergeAt = MergeAt;
    /**
     * MergeExternal() merges tree and another tree into a new tree.
     */
    function MergeExternal(directTree, anotherTree) {
        var newTree = Generic.Clone(directTree);
        MergeInternal(newTree, anotherTree);
        return newTree;
    }
    DirectTree.MergeExternal = MergeExternal;
    /**
     * MergeInternal() merges anotherTree into tree.
     */
    function MergeInternal(directTree, anotherTree) {
        Object.keys(anotherTree).forEach(function (path) {
            var branch = Get(directTree, path);
            var anotherBranch = anotherTree[path];
            if (Generic.IsPlainObject(branch) && Generic.IsPlainObject(anotherBranch)) {
                MergeInternal(branch, anotherBranch);
            }
            else {
                Set(directTree, path, anotherBranch);
            }
        });
    }
    DirectTree.MergeInternal = MergeInternal;
    function OperateLeaf(outTree, directTree, leafOperator) {
        var operandTrees = [];
        for (var _i = 3; _i < arguments.length; _i++) {
            operandTrees[_i - 3] = arguments[_i];
        }
        Object.keys(directTree).forEach(function (key) {
            if (Generic.IsObject(directTree[key])) {
                OperateLeaf.apply(void 0, [outTree[key], directTree[key], leafOperator].concat(operandTrees.map(function (t) { return t[key]; })));
            }
            else {
                RuntimeEnvironment.Set(outTree, key, leafOperator.apply(void 0, [directTree[key]].concat(operandTrees.map(function (t) { return t[key]; }))));
            }
        });
    }
    /**
     * OperateLeafInternal() applys provided operator on each leaf value and save the result to a new tree.
     */
    function OperateLeafExternal(directTree, leafOperator) {
        var operandTrees = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            operandTrees[_i - 2] = arguments[_i];
        }
        var newTree = Generic.Null(directTree);
        OperateLeaf.apply(void 0, [newTree, directTree, leafOperator].concat(operandTrees));
        return newTree;
    }
    DirectTree.OperateLeafExternal = OperateLeafExternal;
    /**
     * OperateLeafInternal() applys provided operator on each leaf value directly in source tree.
     */
    function OperateLeafInternal(directTree, leafOperator) {
        var operandTrees = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            operandTrees[_i - 2] = arguments[_i];
        }
        OperateLeaf.apply(void 0, [directTree, directTree, leafOperator].concat(operandTrees));
    }
    DirectTree.OperateLeafInternal = OperateLeafInternal;
    /**
     * Set() sets the subtree with provided path.
     */
    function Set(directTree, path, value) {
        if (IsFlat(directTree)) {
            RuntimeEnvironment.Set(directTree, path, value);
        }
        else {
            var keys = path.split(DirectTree.DEFAULT_SEPERATOR);
            for (var i = 0; i < keys.length - 1; ++i) {
                var key = keys[i];
                if (directTree[key] == null) {
                    directTree[key] = {};
                }
                directTree = directTree[key];
            }
            RuntimeEnvironment.Set(directTree, keys[keys.length - 1], value);
        }
    }
    DirectTree.Set = Set;
    /**
     * Swap branch of two raw tree.
     */
    function Swap(t1, t2, path) {
        var s1 = Get(t1, path);
        var s2 = Get(t2, path);
        if (s1 != null && s2 != null) {
            Set(t1, path, s2);
            Set(t2, path, s1);
            return true;
        }
        else
            return false;
    }
    DirectTree.Swap = Swap;
})(DirectTree || (DirectTree = {}));

var DirectMath;
(function (DirectMath) {
    /** Extend 1d number function to Numeric Object */
    function Apply1D(f, operand) {
        return DirectTree.OperateLeafExternal(operand, f);
    }
    DirectMath.Apply1D = Apply1D;
    function Abs(operand) {
        return Apply1D(Math.abs, operand);
    }
    DirectMath.Abs = Abs;
    function Acos(operand) {
        return Apply1D(Math.acos, operand);
    }
    DirectMath.Acos = Acos;
    function Acosh(operand) {
        return Apply1D(Math.acosh, operand);
    }
    DirectMath.Acosh = Acosh;
    function Asin(operand) {
        return Apply1D(Math.asin, operand);
    }
    DirectMath.Asin = Asin;
    function Asinh(operand) {
        return Apply1D(Math.asinh, operand);
    }
    DirectMath.Asinh = Asinh;
    function Atan(operand) {
        return Apply1D(Math.atan, operand);
    }
    DirectMath.Atan = Atan;
    function Atanh(operand) {
        return Apply1D(Math.atanh, operand);
    }
    DirectMath.Atanh = Atanh;
    function Cbrt(operand) {
        return Apply1D(Math.cbrt, operand);
    }
    DirectMath.Cbrt = Cbrt;
    function Ceil(operand) {
        return Apply1D(Math.ceil, operand);
    }
    DirectMath.Ceil = Ceil;
    function Cos(operand) {
        return Apply1D(Math.cos, operand);
    }
    DirectMath.Cos = Cos;
    function Cosh(operand) {
        return Apply1D(Math.cosh, operand);
    }
    DirectMath.Cosh = Cosh;
    function Exp(operand) {
        return Apply1D(Math.exp, operand);
    }
    DirectMath.Exp = Exp;
    function Floor(operand) {
        return Apply1D(Math.floor, operand);
    }
    DirectMath.Floor = Floor;
    function Log(operand) {
        return Apply1D(Math.log, operand);
    }
    DirectMath.Log = Log;
    function Log10(operand) {
        return Apply1D(Math.log10, operand);
    }
    DirectMath.Log10 = Log10;
    function Log2(operand) {
        return Apply1D(Math.log2, operand);
    }
    DirectMath.Log2 = Log2;
    function Sin(operand) {
        return Apply1D(Math.sin, operand);
    }
    DirectMath.Sin = Sin;
    function Sinh(operand) {
        return Apply1D(Math.sinh, operand);
    }
    DirectMath.Sinh = Sinh;
    function Sqrt(operand) {
        return Apply1D(Math.sqrt, operand);
    }
    DirectMath.Sqrt = Sqrt;
    function Tan(operand) {
        return Apply1D(Math.tan, operand);
    }
    DirectMath.Tan = Tan;
    function Tanh(operand) {
        return Apply1D(Math.tanh, operand);
    }
    DirectMath.Tanh = Tanh;
    function Trunc(operand) {
        return Apply1D(Math.trunc, operand);
    }
    DirectMath.Trunc = Trunc;
    /** apply nd number function to Numeric Objects */
    function ApplyND(f) {
        var operands = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            operands[_i - 1] = arguments[_i];
        }
        var objectIndex = operands.findIndex(function (o) { return !Generic.IsNumber(o); });
        if (objectIndex > -1) {
            var reference_1 = operands[objectIndex];
            var ops = operands.map(function (o) { return (Generic.IsNumber(o) ? Generic.Assign(reference_1, o) : o); });
            return DirectTree.OperateLeafExternal.apply(DirectTree, [ops[0], f].concat(ops.slice(1)));
        }
        return {};
    }
    function Max() {
        var operands = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            operands[_i] = arguments[_i];
        }
        return ApplyND.apply(void 0, [Math.max].concat(operands));
    }
    DirectMath.Max = Max;
    function Min() {
        var operands = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            operands[_i] = arguments[_i];
        }
        return ApplyND.apply(void 0, [Math.min].concat(operands));
    }
    DirectMath.Min = Min;
    function minmax() {
        var values = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            values[_i] = arguments[_i];
        }
        var min = values[0];
        var max = values[0];
        for (var i = 1; i < values.length; ++i) {
            var val = values[i];
            if (val > max) {
                max = val;
            }
            else if (val < min) {
                min = val;
            }
        }
        return [min, max];
    }
    DirectMath.minmax = minmax;
    function absoluteMax() {
        var values = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            values[_i] = arguments[_i];
        }
        var absolutes = values.map(function (value) { return Math.abs(value); });
        var value = Math.max.apply(Math, absolutes);
        return values[absolutes.indexOf(value)];
    }
    DirectMath.absoluteMax = absoluteMax;
    function AbsoluteMax() {
        var operands = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            operands[_i] = arguments[_i];
        }
        return ApplyND.apply(void 0, [absoluteMax].concat(operands));
    }
    DirectMath.AbsoluteMax = AbsoluteMax;
    function average() {
        var values = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            values[_i] = arguments[_i];
        }
        return sum.apply(void 0, values) / values.length;
    }
    DirectMath.average = average;
    function Average() {
        var operands = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            operands[_i] = arguments[_i];
        }
        return ApplyND.apply(void 0, [average].concat(operands));
    }
    DirectMath.Average = Average;
    function bound(value, min, max) {
        return Math.max(min, Math.min(max, value));
    }
    DirectMath.bound = bound;
    function Bound(operand, min, max) {
        return ApplyND(bound, operand, min, max);
    }
    DirectMath.Bound = Bound;
    function round(value, precision) {
        if (precision === void 0) { precision = 1; }
        return Math.round(value / precision) * precision;
    }
    DirectMath.round = round;
    function Round(operand, precision) {
        if (precision === void 0) { precision = 1; }
        return ApplyND(round, operand, precision);
    }
    DirectMath.Round = Round;
    function sum() {
        var values = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            values[_i] = arguments[_i];
        }
        return values.reduce(function (val1, val2) { return val1 + val2; });
    }
    DirectMath.sum = sum;
    function Sum() {
        var operands = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            operands[_i] = arguments[_i];
        }
        return ApplyND.apply(void 0, [sum].concat(operands));
    }
    DirectMath.Sum = Sum;
    function variance() {
        var values = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            values[_i] = arguments[_i];
        }
        var mean = average.apply(void 0, values);
        return average.apply(void 0, values.map(function (val) { return (val - mean) * (val - mean); }));
    }
    DirectMath.variance = variance;
    function Variance() {
        var operands = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            operands[_i] = arguments[_i];
        }
        return ApplyND.apply(void 0, [variance].concat(operands));
    }
    DirectMath.Variance = Variance;
    function Add() {
        var operands = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            operands[_i] = arguments[_i];
        }
        return ApplyND.apply(void 0, [sum].concat(operands));
    }
    DirectMath.Add = Add;
    function subtract() {
        var values = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            values[_i] = arguments[_i];
        }
        return values.reduce(function (val1, val2) { return val1 - val2; });
    }
    DirectMath.subtract = subtract;
    function Subtract() {
        var operands = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            operands[_i] = arguments[_i];
        }
        return ApplyND.apply(void 0, [subtract].concat(operands));
    }
    DirectMath.Subtract = Subtract;
    function multiply() {
        var values = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            values[_i] = arguments[_i];
        }
        return values.reduce(function (val1, val2) { return val1 * val2; });
    }
    DirectMath.multiply = multiply;
    function Multiply() {
        var operands = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            operands[_i] = arguments[_i];
        }
        return ApplyND.apply(void 0, [multiply].concat(operands));
    }
    DirectMath.Multiply = Multiply;
    function divide() {
        var values = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            values[_i] = arguments[_i];
        }
        return values.reduce(function (val1, val2) { return val1 / val2; });
    }
    DirectMath.divide = divide;
    function Divide() {
        var operands = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            operands[_i] = arguments[_i];
        }
        return ApplyND.apply(void 0, [divide].concat(operands));
    }
    DirectMath.Divide = Divide;
})(DirectMath || (DirectMath = {}));

var DirectFitting = /** @class */ (function () {
    function DirectFitting() {
        this._customFitters = new Map([['singleton', DirectFitting.FitSingleon], ['integer', DirectFitting.FitInteger]]);
        this._customFittables = new Map();
        this._nonfittables = new Set();
    }
    DirectFitting.FitNumeric = function (percentage, start, end) {
        return start + (end - start) * percentage;
    };
    DirectFitting.FitSingleon = function (percentage, start, end) {
        return percentage > 0.5 ? end : start;
    };
    DirectFitting.FitInteger = function (percentage, start, end) {
        return DirectMath.round(start + (end - start) * percentage);
    };
    /**
     * Two object are fittable if and only if none of them contain nonfittable property keys or they contain exactly same nonfittable property key and value.
     */
    DirectFitting.prototype.IsFittable = function (start, end) {
        if (end != null && typeof start === typeof end) {
            if (Generic.IsObject(start) && Generic.IsObject(end)) {
                var fittable_1 = true;
                this._nonfittables.forEach(function (key) {
                    if (fittable_1) {
                        fittable_1 = !start.hasOwnProperty(key) || Generic.Equals(start[key], end[key]);
                    }
                });
                return fittable_1;
            }
            return true;
        }
        return false;
    };
    /**
     * Fit property with consideration of custom fittables.
    */
    DirectFitting.prototype.FitProperty = function (percentage, start, end, property) {
        var fitterKey = this._customFittables.get(property);
        if (fitterKey) {
            var fitter = this._customFitters.get(fitterKey);
            return fitter(percentage, start, end);
        }
        else {
            return this.Fit(percentage, start, end);
        }
    };
    DirectFitting.prototype.Fit = function (percentage, start, end) {
        var _this = this;
        if (Generic.IsArray(start)) {
            if (Generic.IsArray(end) && start.length === end.length) {
                return start.map(function (value, index) { return _this.Fit(percentage, value, end[index]); });
            }
        }
        else if (Generic.IsObject(start)) {
            if (Generic.IsObject(end) && this.IsFittable(start, end)) {
                var fit_1 = Generic.Clone(start);
                DirectMap.Foreach(fit_1, function (_, key) { fit_1[key] = _this.FitProperty(percentage, start[key], end[key], key); });
                return fit_1;
            }
        }
        else if (Generic.IsNumber(start)) {
            if (Generic.IsNumber(end)) {
                return DirectFitting.FitNumeric(percentage, start, end);
            }
        }
        return DirectFitting.FitSingleon(percentage, start, end);
    };
    DirectFitting.prototype.RegisterCustomFitter = function (key, fitter) {
        this._customFitters.set(key, fitter);
    };
    DirectFitting.prototype.RegisterCustomFittables = function (proeprty, fitterKey) {
        this._customFittables.set(proeprty, fitterKey);
    };
    DirectFitting.prototype.RegisterNonFittables = function (property) {
        if (!this._nonfittables.has(property)) {
            this._nonfittables.add(property);
        }
    };
    return DirectFitting;
}());

export { RuntimeEnvironment, Generic, DirectMap, DirectTree, DirectMath, DirectFitting };
//# sourceMappingURL=direct-object.es5.js.map
