"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Generic_1 = require("./Generic");
var DirectMap_1 = require("./DirectMap");
var DirectMath_1 = require("./DirectMath");
var DirectFitting = /** @class */ (function () {
    function DirectFitting() {
        this._customFitters = new Map([['singleton', DirectFitting.FitSingleon], ['integer', DirectFitting.FitInteger]]);
        this._customFittables = new Map();
        this._nonfittables = new Set();
    }
    DirectFitting.FitNumeric = function (percentage, start, end) {
        return start + (end - start) * percentage;
    };
    DirectFitting.FitSingleon = function (percentage, start, end) {
        return percentage > 0.5 ? end : start;
    };
    DirectFitting.FitInteger = function (percentage, start, end) {
        return DirectMath_1.DirectMath.round(start + (end - start) * percentage);
    };
    /**
     * Two object are fittable if and only if none of them contain nonfittable property keys or they contain exactly same nonfittable property key and value.
     */
    DirectFitting.prototype.IsFittable = function (start, end) {
        if (end != null && typeof start === typeof end) {
            if (Generic_1.Generic.IsObject(start) && Generic_1.Generic.IsObject(end)) {
                var fittable_1 = true;
                this._nonfittables.forEach(function (key) {
                    if (fittable_1) {
                        fittable_1 = !start.hasOwnProperty(key) || Generic_1.Generic.Equals(start[key], end[key]);
                    }
                });
                return fittable_1;
            }
            return true;
        }
        return false;
    };
    /**
     * Fit property with consideration of custom fittables.
    */
    DirectFitting.prototype.FitProperty = function (percentage, start, end, property) {
        var fitterKey = this._customFittables.get(property);
        if (fitterKey) {
            var fitter = this._customFitters.get(fitterKey);
            return fitter(percentage, start, end);
        }
        else {
            return this.Fit(percentage, start, end);
        }
    };
    DirectFitting.prototype.Fit = function (percentage, start, end) {
        var _this = this;
        if (Generic_1.Generic.IsArray(start)) {
            if (Generic_1.Generic.IsArray(end) && start.length === end.length) {
                return start.map(function (value, index) { return _this.Fit(percentage, value, end[index]); });
            }
        }
        else if (Generic_1.Generic.IsObject(start)) {
            if (Generic_1.Generic.IsObject(end) && this.IsFittable(start, end)) {
                var fit_1 = Generic_1.Generic.Clone(start);
                DirectMap_1.DirectMap.Foreach(fit_1, function (_, key) { fit_1[key] = _this.FitProperty(percentage, start[key], end[key], key); });
                return fit_1;
            }
        }
        else if (Generic_1.Generic.IsNumber(start)) {
            if (Generic_1.Generic.IsNumber(end)) {
                return DirectFitting.FitNumeric(percentage, start, end);
            }
        }
        return DirectFitting.FitSingleon(percentage, start, end);
    };
    DirectFitting.prototype.RegisterCustomFitter = function (key, fitter) {
        this._customFitters.set(key, fitter);
    };
    DirectFitting.prototype.RegisterCustomFittables = function (proeprty, fitterKey) {
        this._customFittables.set(proeprty, fitterKey);
    };
    DirectFitting.prototype.RegisterNonFittables = function (property) {
        if (!this._nonfittables.has(property)) {
            this._nonfittables.add(property);
        }
    };
    return DirectFitting;
}());
exports.DirectFitting = DirectFitting;
//# sourceMappingURL=DirectFitting.js.map