"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var RuntimeEnvironment_1 = require("./RuntimeEnvironment");
var DirectMap;
(function (DirectMap) {
    /**
     * DoubleMap() creates a new directmap with the results of calling a provided function on every key-value pair in the calling directmap.
    */
    function DoubleMap(directMap, mapping) {
        var newMap = {};
        Object.keys(directMap).forEach(function (oldKey) {
            var _a = mapping(directMap[oldKey], oldKey, directMap), key = _a.key, value = _a.value;
            newMap[key] = value;
        });
        return newMap;
    }
    DirectMap.DoubleMap = DoubleMap;
    function Every(directMap, predicate) {
        return Object.keys(directMap).every(function (key) { return predicate(directMap[key], key, directMap); });
    }
    DirectMap.Every = Every;
    function Filter(directMap, predicate) {
        var resultMap = {};
        Object.keys(directMap).forEach(function (key) {
            if (predicate(directMap[key], key, directMap)) {
                resultMap[key] = directMap[key];
            }
        });
        return resultMap;
    }
    DirectMap.Filter = Filter;
    function Find(directMap, predicate) {
        var key = Object.keys(directMap).find(function (key) { return predicate(directMap[key], key, directMap); });
        if (key) {
            return directMap[key];
        }
        else {
            return undefined;
        }
    }
    DirectMap.Find = Find;
    function FindKey(directMap, predicate) {
        return Object.keys(directMap).find(function (key) { return predicate(directMap[key], key, directMap); });
    }
    DirectMap.FindKey = FindKey;
    function Foreach(directMap, action) {
        Object.keys(directMap).forEach(function (key) {
            action(directMap[key], key, directMap);
        });
    }
    DirectMap.Foreach = Foreach;
    /**
     * From() create a directmap from a value array and a generator function for each seed.
     */
    function From(values, generator) {
        var map = {};
        values.forEach(function (seed, index) {
            var _a = generator(seed, index), key = _a.key, value = _a.value;
            map[key] = value;
        });
        return map;
    }
    DirectMap.From = From;
    /**
     * ValueMap() method creates a new directmap with the results of calling a provided function on every key-value pair in the calling directmap.
     */
    function ValueMap(directMap, mapping) {
        var newMap = {};
        Object.keys(directMap).forEach(function (key) {
            newMap[key] = mapping(directMap[key], key, directMap);
        });
        return newMap;
    }
    DirectMap.ValueMap = ValueMap;
    /**
     * Pack() packs a key and a value to a directmap.
    */
    function Pack(key, value) {
        var pack = {};
        pack[key] = value;
        return pack;
    }
    DirectMap.Pack = Pack;
    function Slice(directMap, keys) {
        return From(keys, function (key) { return ({ key: key, value: directMap[key] }); });
    }
    DirectMap.Slice = Slice;
    function Some(directMap, predicate) {
        return Object.keys(directMap).some(function (key) { return predicate(directMap[key], key, directMap); });
    }
    DirectMap.Some = Some;
    function Splice(directMap, keys) {
        var slice = {};
        keys.forEach(function (key) {
            slice[key] = directMap[key];
            RuntimeEnvironment_1.RuntimeEnvironment.Delete(directMap, key);
        });
        return slice;
    }
    DirectMap.Splice = Splice;
    /**
     * ToMap() converts directmap to es6 map.
    */
    function ToMap(map) {
        var newMap = new Map();
        Foreach(map, function (value, key) { return newMap.set(key, value); });
        return newMap;
    }
    DirectMap.ToMap = ToMap;
})(DirectMap = exports.DirectMap || (exports.DirectMap = {}));
//# sourceMappingURL=DirectMap.js.map