"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Generic;
(function (Generic) {
    /**
     * Find the common value of provided array; for object array, first object is used as key reference, only property with common parts has key in result object.
     */
    function Common(values, eps) {
        if (eps === void 0) { eps = 0; }
        if (values.length > 1) {
            var reference_1 = values[0];
            if (IsObject(reference_1) && !IsArray(reference_1)) {
                if (!values.some(function (value) { return value == null || !IsObject(value); })) {
                    var common_1 = {};
                    Object.keys(reference_1).forEach(function (key) {
                        var subCommon = Common(values.map(function (value) { return value[key]; }), eps);
                        if (subCommon != null) {
                            common_1[key] = subCommon;
                        }
                        else if (values.every(function (value) { return value[key] !== undefined; })) {
                            common_1[key] = null;
                        }
                    });
                    return common_1;
                }
                else
                    return null;
            }
            else {
                return values.every(function (value) { return Equals(value, reference_1, eps); }) ? reference_1 : null;
            }
        }
        else
            return values[0];
    }
    Generic.Common = Common;
    function IsArray(target) {
        return Array.isArray(target);
    }
    Generic.IsArray = IsArray;
    function IsObject(target) {
        return target === Object(target);
    }
    Generic.IsObject = IsObject;
    function IsNumber(target) {
        return typeof target === 'number';
    }
    Generic.IsNumber = IsNumber;
    function IsPlainObject(target) {
        return IsObject(target) && target.constructor.name === 'Object';
    }
    Generic.IsPlainObject = IsPlainObject;
    function IsString(target) {
        return typeof target === 'string';
    }
    Generic.IsString = IsString;
    function IsInstance(target) {
        return IsObject(target) && target.constructor.name !== 'Object';
    }
    Generic.IsInstance = IsInstance;
    /**
     * IsEmpty() determines whether the target is empty (array of zero length, object without any property, string of zero length, null, undefined or class without any members).
     */
    function IsEmpty(target) {
        return target == null || (IsObject(target) && Object.keys(target).length === 0) || (IsString(target) && target.length === 0);
    }
    Generic.IsEmpty = IsEmpty;
    /**
     * Clone() makes a deep/shallow value copy of given object.
     */
    function Clone(target, deep) {
        if (deep === void 0) { deep = true; }
        if (Array.isArray(target)) {
            var clone_1 = target.slice();
            if (deep) {
                clone_1.forEach(function (value, index) {
                    clone_1[index] = Clone(value);
                });
            }
            return clone_1;
        }
        else if (IsObject(target)) {
            // forEach is not called for empty object, so it's logically safe to force a NonEmptyObject guard here.
            var clone_2 = Object.assign({}, target);
            if (deep) {
                Object.entries(target).forEach(function (_a) {
                    var key = _a[0], value = _a[1];
                    clone_2[key] = Clone(value);
                });
            }
            return clone_2;
        }
        else
            return target;
    }
    Generic.Clone = Clone;
    /** Null() return a null version of give object or value. e.g. 1 => null; {a : { b : 1 } } = > { a: { b: null } } */
    function Null(target) {
        return Assign(target, null);
    }
    Generic.Null = Null;
    /** Assign() return version of and index copy of given object or value filled with specified value. e.g. (1, v) => v; ({a : { b : 1 } }, v) = > { a: { b: v } } */
    function Assign(target, value) {
        if (IsObject(target)) {
            if (IsArray(target)) {
                return target.map(function (t) { return Assign(t, value); });
            }
            else if (IsObject(target)) {
                // Agian, forEach is not called for empty object, so I forced a NonEmptyObject guard.
                var nul_1 = {};
                Object.entries(target).forEach(function (_a) {
                    var key = _a[0], subVal = _a[1];
                    nul_1[key] = Assign(subVal, value);
                });
                return nul_1;
            }
        }
        else {
            return value;
        }
    }
    Generic.Assign = Assign;
    /**
     * Equals() does value compare for two arbitrary operands.
     */
    function Equals(o1, o2, eps) {
        if (eps === void 0) { eps = 0; }
        if (IsObject(o1) && IsObject(o2)) {
            if (IsArray(o1)) {
                if (IsArray(o2) && o1.length === o2.length) {
                    return o1.every(function (value, index) { return Equals(value, o2[index]); });
                }
            }
            else {
                return Object.keys(o1).length === Object.keys(o2).length && Object.keys(o1).every(function (key) { return Equals(o1[key], o2[key]); });
            }
        }
        else if (IsNumber(o1) && IsNumber(o2)) {
            return Math.abs(o1 - o2) <= eps;
        }
        return o1 === o2;
    }
    Generic.Equals = Equals;
})(Generic = exports.Generic || (exports.Generic = {}));
//# sourceMappingURL=Generic.js.map