export interface DirectMapEntry<T> {
    key: string;
    value: T;
}
export interface DirectMapObject<T> {
    [name: string]: T;
}
export declare namespace DirectMap {
    type FullMapping<In, Out> = (value: In, key: string, map: DirectMapObject<In>) => DirectMapEntry<Out>;
    type Mapping<In, Out> = (value: In, key: string, map: DirectMapObject<In>) => Out;
    type Predicate<T> = (value: T, key: string, map: DirectMapObject<T>) => boolean;
    type Action<T> = (value: T, key: string, map: DirectMapObject<T>) => void;
    type Generator<In, Out> = (value: In, index: number, array?: In[]) => DirectMapEntry<Out>;
    /**
     * DoubleMap() creates a new directmap with the results of calling a provided function on every key-value pair in the calling directmap.
    */
    function DoubleMap<In, Out>(directMap: DirectMapObject<In>, mapping: FullMapping<In, Out>): DirectMapObject<Out>;
    function Every<T>(directMap: DirectMapObject<T>, predicate: Predicate<T>): boolean;
    function Filter<T>(directMap: DirectMapObject<T>, predicate: Predicate<T>): DirectMapObject<T>;
    function Find<T>(directMap: DirectMapObject<T>, predicate: Predicate<T>): T | undefined;
    function FindKey<T>(directMap: DirectMapObject<T>, predicate: Predicate<T>): string | undefined;
    function Foreach<T>(directMap: DirectMapObject<T>, action: Action<T>): void;
    /**
     * From() create a directmap from a value array and a generator function for each seed.
     */
    function From<In, Out>(values: In[], generator: Generator<In, Out>): DirectMapObject<Out>;
    /**
     * ValueMap() method creates a new directmap with the results of calling a provided function on every key-value pair in the calling directmap.
     */
    function ValueMap<In, Out>(directMap: DirectMapObject<In>, mapping: Mapping<In, Out>): DirectMapObject<Out>;
    /**
     * Pack() packs a key and a value to a directmap.
    */
    function Pack<T>(key: string, value: T): DirectMapObject<T>;
    function Slice<T>(directMap: DirectMapObject<T>, keys: string[]): DirectMapObject<T>;
    function Some<T>(directMap: DirectMapObject<T>, predicate: Predicate<T>): boolean;
    function Splice<T>(directMap: DirectMapObject<T>, keys: string[]): DirectMapObject<T>;
    /**
     * ToMap() converts directmap to es6 map.
    */
    function ToMap<T>(map: DirectMapObject<T>): Map<string, T>;
}
